@@ -109,7 +109,7 @@ class QuantumRegister(Entity):
             address (int): unit address of the quantum register
         """
         index = self.circuit.init_new_qreg_unit()
-        self.units[address]['qubit'] = QuantumState(state=numpy.array([[index]]))
+        self.units[address]['qubit'] = QuantumState(matrix=numpy.array([[index]]))
 
     def reset(self, address: int) -> None:
         r"""Reset the qubit with the given address.
@@ -180,7 +180,7 @@ class QuantumRegister(Entity):
         if self.units[address]['qubit'] is None:
             self.init_unit(address)
 
-        which_qubit = int(self.units[address]['qubit'].state[0][0])
+        which_qubit = int(self.units[address]['qubit'].matrix[0][0])
 
         # Single-qubit gate with condition
         if "condition" in params.keys():
@@ -188,7 +188,7 @@ class QuantumRegister(Entity):
             if operation in ['rx', 'ry', 'rz']:
                 handler = EventHandler(self.circuit, operation, [which_qubit, params['angle']],
                                        signature=self.owner, condition=params['condition'])
-            elif operation == 'u3':
+            elif operation in ['u', 'u3']:
                 handler = EventHandler(self.circuit, operation, [which_qubit, *params['angles']],
                                        signature=self.owner, condition=params['condition'])
             # Fixed single-qubit gate with condition
@@ -199,7 +199,7 @@ class QuantumRegister(Entity):
             # Parameterized single-qubit gate without condition
             if operation in ['rx', 'ry', 'rz']:
                 handler = EventHandler(self.circuit, operation, [which_qubit, params['angle']], signature=self.owner)
-            elif operation == 'u3':
+            elif operation in ['u', 'u3']:
                 handler = EventHandler(self.circuit, operation, [which_qubit, *params['angles']], signature=self.owner)
             # Fixed single-qubit gate without condition (including measurement)
             else:
@@ -221,12 +221,12 @@ class QuantumRegister(Entity):
         if self.units[address[1]]['qubit'] is None:
             self.init_unit(address[1])
 
-        ctrl, targ = int(self.units[address[0]]['qubit'].state[0][0]), int(self.units[address[1]]['qubit'].state[0][0])
+        ctrl, targ = int(self.units[address[0]]['qubit'].matrix[0][0]), int(self.units[address[1]]['qubit'].matrix[0][0])
 
         # Parameterized double-qubit gate
         if operation in ['crx', 'cry', 'crz']:
             handler = EventHandler(self.circuit, operation, [[ctrl, targ], params['angle']], signature=self.owner)
-        elif operation == 'cu3':
+        elif operation in ['cu', 'cu3']:
             handler = EventHandler(self.circuit, operation, [[ctrl, targ], *params['angles']], signature=self.owner)
         # Fixed double-qubit gate
         else:
@@ -324,13 +324,27 @@ class QuantumRegister(Entity):
         """
         self.__add_single_qubit_gate('rz', address, angle=theta, **condition)
 
+    def u(self, address: int, theta: Union[float, int], phi: Union[float, int], gamma: Union[float, int],
+          **condition) -> None:
+        r"""Apply a rotation gate on the local qubit with a given address.
+
+        Args:
+            address (int): local address of the qubit to act on in the quantum register
+            theta (Union[float, int]): the rotation angle of the Rx gate
+            phi (Union[float, int]): the rotation angle of the left Rz gate
+            gamma (Union[float, int]): the rotation angle of the right Rz gate
+            **condition (int): control qubit address
+        """
+        angles = [theta, phi, gamma]
+        self.__add_single_qubit_gate('u', address, angles=angles, **condition)
+
     def u3(self, address: int, theta: Union[float, int], phi: Union[float, int], gamma: Union[float, int],
            **condition) -> None:
         r"""Apply a rotation gate on the local qubit with a given address.
 
         Args:
             address (int): local address of the qubit to act on in the quantum register
-            theta (Union[float, int]): the rotation angle of the Rx gate
+            theta (Union[float, int]): the rotation angle of the Ry gate
             phi (Union[float, int]): the rotation angle of the left Rz gate
             gamma (Union[float, int]): the rotation angle of the right Rz gate
             **condition (int): control qubit address
@@ -405,13 +419,26 @@ class QuantumRegister(Entity):
         """
         self.__add_double_qubit_gate('crz', address, angle=theta)
 
+    def cu(self, address: List[int],
+           theta: Union[float, int], phi: Union[float, int], gamma: Union[float, int]) -> None:
+        r"""Apply a Controlled-rotation gate on the local qubit with given addresses.
+
+        Args:
+            address (List[int]): local addresses of the qubits to act on in the quantum register
+            theta (Union[float, int]): the rotation angle of the Rx gate
+            phi (Union[float, int]): the rotation angle of the left Rz gate
+            gamma (Union[float, int]): the rotation angle of the right Rz gate
+        """
+        angles = [theta, phi, gamma]
+        self.__add_double_qubit_gate('cu', address, angles=angles)
+
     def cu3(self, address: List[int],
             theta: Union[float, int], phi: Union[float, int], gamma: Union[float, int]) -> None:
         r"""Apply a Controlled-rotation gate on the local qubit with given addresses.
 
         Args:
             address (List[int]): local addresses of the qubits to act on in the quantum register
-            theta (Union[float, int]): the rotation angle of the Rx gate
+            theta (Union[float, int]): the rotation angle of the Ry gate
             phi (Union[float, int]): the rotation angle of the left Rz gate
             gamma (Union[float, int]): the rotation angle of the right Rz gate
         """
@@ -442,7 +469,7 @@ class QuantumRegister(Entity):
             self.__add_single_qubit_gate("measure", address)
 
         # Side effect for measurement, the trick here is to use the qubit register as its measurement outcome
-        which_qubit = int(self.units[address]['qubit'].state[0][0])
+        which_qubit = int(self.units[address]['qubit'].matrix[0][0])
         self.units[address]['outcome'] = which_qubit
 
         # Rest the unit once measured
