@@ -86,19 +86,9 @@ class KeyGeneration(Protocol, ABC):
         if kwargs['peer'] == self.peer:
             self.start(**kwargs)
 
-    def receive_quantum_msg(self, msg: "QuantumMessage", **kwargs) -> None:
-        r"""Receive a quantum message from the node.
-
-        Args:
-            msg (Message): received quantum message
-            **kwargs (Any): keyword arguments for receiving the quantum message
-        """
-        self.node.polar_detector.receive(msg)
-
     def finish(self) -> None:
         r"""Finish the key generation protocol.
         """
-        from qcompute_qnet.models.qkd.routing import QKDRouting
         self.node.env.logger.info(f"{self.node.name} finished key generation with {self.peer.name}")
         self.node.env.logger.info(f"{self.node.name}'s sifted keys (with {self.peer.name} in decimal): "
                                   f"{self.sifted_keys}")
@@ -109,7 +99,11 @@ class KeyGeneration(Protocol, ABC):
             self.node.keys[self.peer] = self.sifted_keys
         else:
             # TODO: only consider QKDRouting as an upper protocol in this version
-            handler = EventHandler(self, "send_upper", [QKDRouting], peer=self.peer, sifted_keys=self.sifted_keys)
+            qkd_routing = type(self.upper_protocols[-1])
+            towards = 'up' if self.role == PrepareAndMeasure.Role.TRANSMITTER else 'down'
+
+            handler = EventHandler(self, "send_upper", [qkd_routing],
+                                   peer=self.peer, sifted_keys=self.sifted_keys, towards=towards)
             self.node.scheduler.schedule_now(handler)
 
 
@@ -181,7 +175,7 @@ class PrepareAndMeasure(KeyGeneration, ABC):
                 raise TypeError(f"Setting {attr} is not allowed in {self.name}")
 
     def receive_classical_msg(self, msg: "ClassicalMessage", **kwargs) -> None:
-        r"""Node receives a classical message.
+        r"""Receive a classical message from the node.
 
         Args:
             msg (ClassicalMessage): classical message used for the prepare-and-measure protocol
@@ -195,6 +189,20 @@ class PrepareAndMeasure(KeyGeneration, ABC):
             elif self.role == PrepareAndMeasure.Role.RECEIVER:
                 self.receiver_receive_classical_msg(msg)
 
+    def receive_quantum_msg(self, msg: "QuantumMessage", **kwargs) -> None:
+        r"""Receive a quantum message from the node.
+
+        Args:
+            msg (Message): received quantum message
+            **kwargs (Any): keyword arguments for receiving the quantum message
+        """
+        if self.role == PrepareAndMeasure.Role.RECEIVER:
+            if isinstance(self.node.polar_detector, dict):
+                src = kwargs['src']
+                self.node.polar_detector[src].receive(msg)
+            else:
+                self.node.polar_detector.receive(msg)
+
     @abstractmethod
     def transmitter_receive_classical_msg(self, msg: "ClassicalMessage", **kwargs) -> None:
         r"""Transmitter receives a classical message from the receiver.
@@ -215,7 +223,7 @@ class PrepareAndMeasure(KeyGeneration, ABC):
         """
         pass
 
-    def receiver_get_bits(self) -> List:
+    def receiver_get_bits(self, **kwargs) -> List:
         r"""Get bits from the detectors' time records.
 
         Warnings:
@@ -224,13 +232,20 @@ class PrepareAndMeasure(KeyGeneration, ABC):
             However, this makes no much difference to the key rate's estimation.
             Users can override this method if necessary.
 
+        Args:
+            **kwargs: keyword argument to specify the polar detector for different nodes
+
         Returns:
             list: receiver's bits list
         """
         bits = [None] * self.pulse_num
-
-        records_zero = self.node.polar_detector.detectors[0].pop_records()
-        records_one = self.node.polar_detector.detectors[1].pop_records()
+        if isinstance(self.node.polar_detector, dict):
+            peer = kwargs['src']
+            records_zero = self.node.polar_detector[peer].detectors[0].pop_records()
+            records_one = self.node.polar_detector[peer].detectors[1].pop_records()
+        else:
+            records_zero = self.node.polar_detector.detectors[0].pop_records()
+            records_one = self.node.polar_detector.detectors[1].pop_records()
 
         for record in records_zero:
             index = record[0]
@@ -261,7 +276,7 @@ class PrepareAndMeasure(KeyGeneration, ABC):
         """
         # Find the peer protocol
         for proto in self.peer.protocol_stack.protocols:
-            if isinstance(proto, KeyGeneration) and proto.peer == self.node:
+            if isinstance(proto, KeyGeneration) and proto.peer == self.node and proto.role != self.role:
                 peer_proto = proto
 
         # Calculate error rate of all sifted keys
@@ -367,20 +382,22 @@ class BB84(PrepareAndMeasure):
         and prepares to emit photon pulses.
         """
         self.node.env.logger.info(f"{self.node.name} started BB84 protocol with {self.peer.name}")
+        # Select the corresponding photon source for key generation
+        photon_source = self.node.photon_source[self.peer] if isinstance(self.node.photon_source, dict)\
+            else self.node.photon_source
 
-        self.pulse_num = round(self.key_length / self.node.photon_source.mean_photon_num)
-        self.light_duration = round(self.pulse_num / self.node.photon_source.frequency * 1e12)
-        self.emission_rest = round(1e12 / self.node.photon_source.frequency)
+        self.pulse_num = round(self.key_length / photon_source.mean_photon_num)
+        self.light_duration = round(self.pulse_num / photon_source.frequency * 1e12)
+        self.emission_rest = round(1e12 / photon_source.frequency)
 
         transmitter_emit_time = self.node.env.now + self.node.cchannel(self.peer).delay
         clock = transmitter_emit_time + self.node.qchannel(self.peer).delay  # clock time for photon reception
-
         start_msg = BB84.Message(src=self.node,
                                  dst=self.peer,
                                  protocol=BB84,
                                  data={'type': BB84.Message.Type.START,
                                        'clock': clock,
-                                       'source_frequency': self.node.photon_source.frequency,
+                                       'source_frequency': photon_source.frequency,
                                        'light_duration': self.light_duration,
                                        'emission_rest': self.emission_rest})
         self.node.send_classical_msg(dst=self.peer, msg=start_msg)
@@ -411,7 +428,10 @@ class BB84(PrepareAndMeasure):
                 # Prepare states |0>, |1>, |+>, |-> (in the form of density matrices) for photon emission
                 states.append(basis[bits[i]] @ np.conjugate(basis[bits[i]]).T)
 
-            handler = EventHandler(self.node.photon_source, "emit", [self.peer, states])
+            photon_source = self.node.photon_source[self.peer] if isinstance(self.node.photon_source, dict) \
+                else self.node.photon_source
+
+            handler = EventHandler(photon_source, "emit", [self.peer, states])
             self.scheduler.schedule_now(handler)
 
             # Schedule next round of photon emission
@@ -425,12 +445,16 @@ class BB84(PrepareAndMeasure):
         Receiver pops the measurement outcomes and resets the measurement bases to
         prepare for receiving photon pulses in the next round.
         """
+        # Select the corresponding detector for key generation
+        polar_detector = self.node.polar_detector[self.peer] if isinstance(self.node.polar_detector, dict) \
+            else self.node.polar_detector
+
         if self.status == KeyGeneration.Status.WORKING:
             self.round += 1
             self.node.env.logger.debug(f"{self.node.name} finished receiving photons from {self.peer.name}"
                                        f" (Round {self.round})")
 
-            self.rx_bits_list.append(self.receiver_get_bits())
+            self.rx_bits_list.append(self.receiver_get_bits(src=self.peer))
             # Reset measurement bases for the next round of photon reception
             new_bases = list(np.random.choice([0, 1], self.pulse_num, p=self.rx_bases_ratio))
             self.rx_bases_list.append(new_bases)
@@ -438,13 +462,13 @@ class BB84(PrepareAndMeasure):
 
             new_bases = [self.basis_encoding[basis] for basis in new_bases]
             # Reset parameters for polarization detector
-            self.node.polar_detector.set_beamsplitter(clock=clock, bases=new_bases)
+            polar_detector.set_beamsplitter(clock=clock, bases=new_bases)
 
             # Schedule next round of stopping photon reception
             handler = EventHandler(self, "receiver_stop_reception")
             self.scheduler.schedule_at(clock + self.light_duration, handler)
 
-            # Ask the transmitter to announce the bases for key sifting
+            # Ask the transmitter to announce the bases for key generation
             request_bases_msg = BB84.Message(src=self.node,
                                              dst=self.peer,
                                              protocol=BB84,
@@ -453,8 +477,8 @@ class BB84(PrepareAndMeasure):
             self.node.send_classical_msg(dst=self.peer, msg=request_bases_msg)
 
         elif self.status == KeyGeneration.Status.DONE:
-            self.node.polar_detector.detectors[0].turn_off()
-            self.node.polar_detector.detectors[1].turn_off()
+            polar_detector.detectors[0].turn_off()
+            polar_detector.detectors[1].turn_off()
 
     def transmitter_receive_classical_msg(self, msg: "BB84.Message", **kwargs) -> None:
         r"""Transmitter receives a BB84 message from the receiver.
@@ -463,7 +487,16 @@ class BB84(PrepareAndMeasure):
             msg (BB84.Message): message for BB84 protocol
             **kwargs (Any): keyword arguments for receiving the classical message
         """
-        photon_round = msg.data['round']
+        # Filter the message that is not intended for this instance
+        if msg.data['type'].name not in ['MATCHING', 'BASES_REQUEST']:
+            return
+
+        photon_round = msg.data.get('round')
+
+        # Check photon round to filter the outdated messages resulting from BB84 protocol's reusing
+        if photon_round is not None and self.round < photon_round:
+            self.node.env.logger.debug(f"Message with round {photon_round} was discarded at local round {self.round}")
+            return
 
         if msg.data['type'] == BB84.Message.Type.BASES_REQUEST:
             self.node.env.logger.debug(f"{self.node.name} received bases request from"
@@ -471,6 +504,7 @@ class BB84(PrepareAndMeasure):
 
             # Pop the earliest bases and send to the receiver
             bases = self.tx_bases_list.pop(0)
+
             bases_msg = BB84.Message(src=self.node,
                                      dst=self.peer,
                                      protocol=BB84,
@@ -511,6 +545,10 @@ class BB84(PrepareAndMeasure):
             msg (BB84.Message): message for BB84 protocol
             **kwargs (Any): keyword arguments for receiving the classical message
         """
+        # Filter the message that is not intended for this instance
+        if msg.data['type'].name not in ['START', 'BASES']:
+            return
+
         if msg.data['type'] == BB84.Message.Type.START:
             self.node.env.logger.debug(f"{self.node.name} received message 'START' from {self.peer.name}")
 
@@ -526,9 +564,12 @@ class BB84(PrepareAndMeasure):
 
             bases = [self.basis_encoding[basis] for basis in bases]  # map the bits to bases for measurement
             # Set parameters for polarization detector
-            self.node.polar_detector.set_beamsplitter(clock=clock, frequency=frequency, bases=bases)
-            self.node.polar_detector.detectors[0].turn_on()
-            self.node.polar_detector.detectors[1].turn_on()
+            polar_detector = self.node.polar_detector[self.peer] if isinstance(self.node.polar_detector, dict)\
+                else self.node.polar_detector
+
+            polar_detector.set_beamsplitter(clock=clock, frequency=frequency, bases=bases)
+            polar_detector.detectors[0].turn_on()
+            polar_detector.detectors[1].turn_on()
 
             # Stop receiving photons after a light duration
             handler = EventHandler(self, "receiver_stop_reception")
@@ -536,6 +577,13 @@ class BB84(PrepareAndMeasure):
 
         elif msg.data['type'] == BB84.Message.Type.BASES:
             photon_round = msg.data['round']
+
+            # Check photon round to filter the outdated messages resulting from BB84 protocol's reusing
+            if self.round < photon_round:
+                self.node.env.logger.debug(f"Message with round {photon_round} was discarded"
+                                           f" at local round {self.round}.")
+                return
+
             self.node.env.logger.debug(f"{self.node.name} received bases from {self.peer.name} "
                                        f"(Round: {photon_round})")
 
