@@ -29,7 +29,6 @@ from numpy import reshape, pi, conj, real, random, sqrt
 from qcompute_qnet import EPSILON
 from qcompute_qnet.quantum.basis import Basis
 from qcompute_qnet.quantum.gate import Gate
-from qcompute_qnet.quantum.mcalculus import transpile_to_pattern
 from qcompute_qnet.quantum.pattern import Pattern
 from qcompute_qnet.quantum.state import PureState, Plus
 from qcompute_qnet.quantum.utils import kron, print_progress
@@ -162,7 +161,7 @@ class MBQC:
             raise ArgumentTypeError(f"Input {pattern} is not a standard EMC pattern.")
 
         # Set graph according to entanglement commands
-        edges = [tuple(cmd.which_qubits) for cmd in cmds if cmd.name == 'E']
+        edges = [tuple(cmd.which_qubit) for cmd in cmds if cmd.name == 'E']
         vertices = list(set([vertex for edge in edges for vertex in list(edge)]))
         graph = [vertices, edges]
         self.set_graph(graph)
@@ -210,7 +209,7 @@ class MBQC:
             state_vector = numpy.array([[1]])
             systems = []
         else:
-            state_vector = state.state
+            state_vector = state.matrix
             # If a pattern is set, map the input state systems to the pattern's input
             if self.__pattern is not None:
                 for label in state.systems:
@@ -271,18 +270,18 @@ class MBQC:
                                         f"Control qubit must not be the same as target qubit.")
 
             # Find the control and target qubits and permute them to the front
-            self.__bg_state.permute_to_front(qubit1)
-            self.__bg_state.permute_to_front(qubit2)
+            self.__bg_state.substates[0].permute_to_front(qubit1)
+            self.__bg_state.substates[0].permute_to_front(qubit2)
 
             new_state = self.__bg_state
             new_state_len = new_state.length
             qua_length = int(new_state_len / 4)
             # Reshape the state, apply CZ and reshape it back
-            new_state.state = reshape(Gate.CZ() @ reshape(new_state.state, [4, qua_length]), [new_state_len, 1])
+            new_state.substates[0].matrix = reshape(Gate.CZ() @ reshape(new_state.matrix, [4, qua_length]), [new_state_len, 1])
 
             # Update the order of active vertices and the background state
             self.vertex.active = new_state.systems
-            self.__bg_state = PureState(new_state.state, new_state.systems)
+            self.__bg_state = PureState(new_state.matrix, new_state.systems)
 
     def __apply_pauli_gate(self, gate: str, which_qubit: Any) -> None:
         r"""Apply Pauli gate.
@@ -291,7 +290,7 @@ class MBQC:
             gate (str): name of the Pauli gate
             which_qubit (Any): qubit to manipulate
         """
-        self.__bg_state.permute_to_front(which_qubit)
+        self.__bg_state.substates[0].permute_to_front(which_qubit)
         new_state = self.__bg_state
         new_state_len = new_state.length
         half_length = int(new_state_len / 2)
@@ -304,10 +303,11 @@ class MBQC:
             raise ArgumentTypeError(f"Input {gate} should be string 'X' or 'Z'.")
 
         # Reshape the state, apply X and reshape it back
-        new_state.state = reshape(gate_mat @ reshape(new_state.state, [2, half_length]), [new_state_len, 1])
+        new_state.substates[0].matrix = \
+            reshape(gate_mat @ reshape(new_state.matrix, [2, half_length]), [new_state_len, 1])
         # Update the order of active vertices and the background state
         self.vertex.active = new_state.systems
-        self.__bg_state = PureState(new_state.state, new_state.systems)
+        self.__bg_state = PureState(new_state.matrix, new_state.systems)
 
     def __create_graph_state(self, which_qubit: Any) -> None:
         r"""Create a graph state based on the current qubit to measure.
@@ -327,8 +327,8 @@ class MBQC:
         self.vertex.pending = list(set(self.vertex.pending).difference(self.vertex.active))
 
         # Compute the new background state vector
-        new_bg_state_vector = kron([self.__bg_state.state] +
-                                   [self.__vertex_to_state[vertex].state for vertex in append_qubits])
+        new_bg_state_vector = kron([self.__bg_state.matrix] +
+                                   [self.__vertex_to_state[vertex].matrix for vertex in append_qubits])
 
         # Update the background state and apply cz
         self.__bg_state = PureState(new_bg_state_vector, self.vertex.active)
@@ -348,7 +348,7 @@ class MBQC:
         if which_qubit not in self.vertex.active:
             raise ArgumentTypeError(f"Invalid qubit: ({which_qubit})! The qubit must be activated before measurement.")
 
-        self.__bg_state.permute_to_front(which_qubit)
+        self.__bg_state.substates[0].permute_to_front(which_qubit)
         new_bg_state = self.__bg_state
         self.vertex.active = new_bg_state.systems
         half_length = int(new_bg_state.length / 2)
@@ -360,7 +360,7 @@ class MBQC:
         for result in [0, 1]:
             basis_dag = conj(basis[result]).T
             # Reshape the state, multiply the basis and reshape it back
-            state_unnorm[result] = reshape(basis_dag @ reshape(new_bg_state.state, [2, half_length]), [half_length, 1])
+            state_unnorm[result] = reshape(basis_dag @ reshape(new_bg_state.matrix, [2, half_length]), [half_length, 1])
             probability = conj(state_unnorm[result]).T @ state_unnorm[result]
             prob[result] = real(probability) if probability.dtype.name == 'COMPLEX128' else probability
 
@@ -502,7 +502,7 @@ class MBQC:
                 self.__create_graph_state(qubit)
                 self.vertex.measured.append(qubit)
                 self.max_active = max(len(self.vertex.active), self.max_active)
-                self.__bg_state = PureState(self.__bg_state.state, self.vertex.active)
+                self.__bg_state = PureState(self.__bg_state.matrix, self.vertex.active)
 
         self.vertex.measured = measured_qubits  # restore the measured qubits
 
@@ -534,7 +534,7 @@ class MBQC:
         # The output state's label is messy (e.g. [(2, 0), (0, 1), (1, 3)...]),
         # so we permute the systems in order
         q_output = self.__pattern.output_[1]
-        self.__status.permute_systems(q_output)
+        self.__status.substates[0].permute_systems(q_output)
         self.__bg_state = self.__status
 
         self.__update_history()
@@ -571,8 +571,6 @@ class MBQC:
             row2qubit = self.__map_qubit_to_row(output_lst)
 
             # Mark the classical outputs with their measurement outcomes
-            # Mark a quantum register with a place holder '_'
-            # bit_lst = [str(self.__outcome[row2qubit[i]]) if row2qubit[i] in c_output else '_' for i in range(width)]
             bit_lst = [str(self.__outcome[row2qubit[i]]) if row2qubit[i] in c_output else '' for i in range(width)]
             bit_str = ''.join(bit_lst)
             return bit_str
@@ -707,13 +705,12 @@ class MBQC:
         self.__track = track
 
 
-def run_circuit(circuit: "Circuit", shots=1024, input_state=None, optimize=True) -> dict:
+def run_circuit(circuit: "Circuit", shots=1024, input_state=None, optimize=None) -> dict:
     r"""Run a quantum circuit by its equivalent MBQC model.
 
     Note:
         This method transpiles a quantum circuit to its equivalent MBQC model first
         and then runs the MBQC pattern to get equivalent sampling result.
-        It can work well for large-scale quantum shallow circuits with the option ``optimize=True``.
 
     Warnings:
         We should check if the circuit has sequential registers first.
@@ -723,7 +720,7 @@ def run_circuit(circuit: "Circuit", shots=1024, input_state=None, optimize=True)
         circuit (Circuit): quantum circuit to run
         shots (int, optional): number of sampling
         input_state (PureState, optional): input quantum state
-        optimize (bool): whether to optimize the measurement order
+        optimize (str): optimize the measurement order
 
     Returns:
         dict: classical results
@@ -736,25 +733,14 @@ def run_circuit(circuit: "Circuit", shots=1024, input_state=None, optimize=True)
 
     # Add a layer of H gates to start from zero states
     if input_state is None:
-        h_layer = [{"name": 'h', "which_qubit": [i], "params": None} for i in range(circuit.width)]
+        h_layer = [{"name": 'h', "which_qubit": [i]} for i in range(circuit.width)]
         new_circuit = circuit.copy()
         new_circuit._history = h_layer + circuit.gate_history
         circuit = new_circuit
 
-    pattern = transpile_to_pattern(circuit, shift_signal=True, optimize=optimize, track=False)
-
-    samples = []
-
-    for shot in range(shots):
-        mbqc = MBQC()
-        mbqc.set_pattern(pattern)
-        mbqc.set_input_state(input_state)
-        mbqc.run_pattern()
-        c_output = mbqc.get_classical_output()
-        samples.append(c_output)
-
-    sample_dict = {}
-    for string in list(set(samples)):
-        sample_dict[string] = samples.count(string)
+    circuit.print_list()
+    pattern = circuit.to_pattern(shift_signal=True, optimize=optimize, track=False)
+    results = pattern.run(shots=shots, input_state=input_state)
+    sample_dict = results['counts']
 
     return sample_dict
