@@ -36,6 +36,8 @@ DEFAULT_BACKEND = 'local_baidu_sim2'
 DEFAULT_TOKEN = None
 QCompute.Define.Settings.outputInfo = False
 QCompute.Define.Settings.drawCircuitControl = []
+QCompute.Define.Settings.autoClearOutputDirAfterFetchMeasure = True
+QCompute.Define.Settings.cloudTaskDoNotWriteFile = True
 
 
 def set_qcompute_default_backend(backend: str) -> None:
@@ -101,9 +103,9 @@ def run_circuit(circuit: "Circuit", shots: int, backend=None, token=None) -> dic
         from QCompute is given by ``{'01': 5, '00': 3}``, then our return gives ``{'10': 5, '00': 3}``.
     """
     single_qubit_gates = {
-        'id': QCompute.ID, 's': QCompute.S, 't': QCompute.T,
+        'id': QCompute.ID, 's': QCompute.S, 'sdg': QCompute.SDG, 't': QCompute.T, 'tdg': QCompute.TDG,
         'h': QCompute.H, 'x': QCompute.X, 'y': QCompute.Y, 'z': QCompute.Z,
-        'rx': QCompute.RX, 'ry': QCompute.RY, 'rz': QCompute.RZ, 'u3': QCompute.U,
+        'rx': QCompute.RX, 'ry': QCompute.RY, 'rz': QCompute.RZ, 'u3': QCompute.U, 'u': None,
         'm': QCompute.MeasureZ
     }
     multi_qubits_gates = {
@@ -140,6 +142,10 @@ def run_circuit(circuit: "Circuit", shots: int, backend=None, token=None) -> dic
                 gate_func(gate['angle'])(qcompute_env.Q[which_qubit[0]])
             elif name in ['u3']:
                 gate_func(*gate['angles'])(qcompute_env.Q[which_qubit[0]])
+            elif name in ['u']:
+                QCompute.S(qcompute_env.Q[which_qubit[0]])
+                QCompute.U(*gate['angles'])(qcompute_env.Q[which_qubit[0]])
+                QCompute.SDG(qcompute_env.Q[which_qubit[0]])
             elif name in ['m']:
                 gate_func([qcompute_env.Q[which_qubit[0]]], [which_qubit[0]])
 
