@@ -22,14 +22,13 @@ This will be used to manipulate the measurement patterns in the measurement-base
 """
 
 from argparse import ArgumentTypeError
-from typing import List, Any
+from typing import List, Tuple
 from numpy import pi
 from qcompute_qnet.quantum.pattern import Pattern
 from qcompute_qnet.quantum.utils import print_progress
 
 __all__ = [
-    "MCalculus",
-    "transpile_to_pattern"
+    "MCalculus"
 ]
 
 
@@ -43,13 +42,13 @@ class MCalculus:
     def __init__(self):
         r"""Constructor for MCalculus class.
         """
-        self.__circuit = None  # circuit
-        self.__circuit_width = None  # circuit width
-        self.__circuit_slice = []  # sliced circuit
-        self.__wild_pattern = []  # wild pattern
-        self.__pattern = None  # standard pattern
-        self.__measured_qubits = []  # measured qubits
-        self.__track = False  # progress tracking
+        self._circuit = None  # circuit
+        self._circuit_width = None  # circuit width
+        self._circuit_slice = []  # sliced circuit
+        self._wild_pattern = []  # wild pattern
+        self._pattern = None  # standard pattern
+        self._measured_qubits = []  # measured qubits
+        self._track = False  # progress tracking
 
     def set_circuit(self, circuit: "Circuit") -> None:
         r"""Set a quantum circuit.
@@ -57,8 +56,8 @@ class MCalculus:
         Args:
             circuit (Circuit): quantum circuit to set
         """
-        self.__circuit = circuit
-        self.__circuit_width = circuit.width
+        self._circuit = circuit
+        self._circuit_width = circuit.width
 
     def __slice_circuit(self, gate_history: list) -> None:
         r"""Slice a quantum circuit and label the input and output qubits for each gate.
@@ -78,7 +77,7 @@ class MCalculus:
 
             if qubit_number == 1:  # single qubit gate
                 which_qubit = which_qubits[0]  # take the qubit index
-                if which_qubit in self.__measured_qubits:
+                if which_qubit in self._measured_qubits:
                     raise ArgumentTypeError(f"Invalid qubit index: ({which_qubit})!\n"
                                             f"Please check your qubit index as this qubit has already been measured.")
 
@@ -94,10 +93,10 @@ class MCalculus:
             else:  # double qubit gate
                 control = which_qubits[0]  # take the control qubit
                 target = which_qubits[1]  # take the target qubit
-                if control in self.__measured_qubits:
+                if control in self._measured_qubits:
                     raise ArgumentTypeError(f"Invalid qubit index: ({control})!\n"
                                             f"Please check your qubit index as this qubit has already been measured.")
-                if target in self.__measured_qubits:
+                if target in self._measured_qubits:
                     raise ArgumentTypeError(f"Invalid qubit index: ({target})!\n"
                                             f"Please check your qubit index as this qubit has already been measured.")
 
@@ -118,7 +117,7 @@ class MCalculus:
 
             # The gate after slicing has a form: [original_gate, input_, output_]
             # E.g. = [[cx, [0, 1], None], input_, output_]
-            self.__circuit_slice.append([gate, input_, output_])
+            self._circuit_slice.append([gate, input_, output_])
 
     @staticmethod
     def __set_ancilla(input_: list, output_: list, ancilla_number_list=None) -> list:
@@ -130,7 +129,7 @@ class MCalculus:
         Args:
             input_ (list): input vertices
             output_ (list): output vertices
-            ancilla_number_list (list): number of ancilla qubits
+            ancilla_number_list (list, optional): number of ancilla qubits
 
         Returns:
             list: ancilla qubit labels
@@ -292,8 +291,8 @@ class MCalculus:
             Z5 = Pattern.CommandZ(output_[0], [ancilla[1]])
             commands = [E12, E23, E34, E45, M1, M2, M3, M4, X5, Z5]
 
-        elif name == 'u3':  # single-qubit unitary gate
-            # Note: U = Rz(\phi) Rx(\theta) Rz(\lamda)
+        elif name == 'u':  # single-qubit unitary gate
+            # Note: U = Rz(\phi) Rx(\theta) Rz(\gamma)
             ancilla = self.__set_ancilla(input_, output_, [3])
             theta, phi, gamma = original_gate["angles"]
             E12 = Pattern.CommandE([input_[0], ancilla[0]])
@@ -308,8 +307,8 @@ class MCalculus:
             Z5 = Pattern.CommandZ(output_[0], [ancilla[1]])
             commands = [E12, E23, E34, E45, M1, M2, M3, M4, X5, Z5]
 
-        elif name == 'u':  # single-qubit unitary gate
-            # Note: U = Rz(\phi) Ry(\theta) Rz(\lamda) usually used in MBQC
+        elif name == 'u3':  # single-qubit unitary gate
+            # Note: U3 = Rz(\phi) Ry(\theta) Rz(\gamma) usually used in MBQC
             ancilla = self.__set_ancilla(input_, output_, [3])
             theta, phi, gamma = original_gate["angles"]
             E12 = Pattern.CommandE([input_[0], ancilla[0]])
@@ -336,71 +335,26 @@ class MCalculus:
             Z4 = Pattern.CommandZ(output_[1], [input_[1]])
             commands = [E23, E13, E34, M2, M3, X4, Z1, Z4]
 
-        # Measurement pattern of CNOT with 15 qubits, c.f. [arXiv: quant-ph/0301052v2]
-        # Note: due to the '1' in byproduct Z of qubit 7, we manually add a Z gate after qubit 7 to eliminate this
-        elif name == 'cnot15':  # control not gate
-            input1, input2 = input_
-            output1, output2 = output_
-            ancilla = self.__set_ancilla(input_, output_, [7, 5])
-            new_row = (input2[0] + input1[0]) / 2
-            new_col_1 = output1[1] + input1[1]
-            new_col_2 = output2[1] + input2[1]
-            new_col = (new_col_1 + new_col_2) / 4
-            ancilla.append((new_row, new_col))
-            E12 = Pattern.CommandE([input1, ancilla[0]])
-            E23 = Pattern.CommandE([ancilla[0], ancilla[1]])
-            E34 = Pattern.CommandE([ancilla[1], ancilla[2]])
-            E45 = Pattern.CommandE([ancilla[2], ancilla[3]])
-            E48 = Pattern.CommandE([ancilla[2], ancilla[12]])
-            E56 = Pattern.CommandE([ancilla[3], ancilla[4]])
-            E67 = Pattern.CommandE([ancilla[4], ancilla[5]])
-            E716 = Pattern.CommandE([ancilla[5], ancilla[6]])
-            E1617 = Pattern.CommandE([ancilla[6], output1])
-            E910 = Pattern.CommandE([input2, ancilla[7]])
-            E1011 = Pattern.CommandE([ancilla[7], ancilla[8]])
-            E1112 = Pattern.CommandE([ancilla[8], ancilla[9]])
-            E812 = Pattern.CommandE([ancilla[12], ancilla[9]])
-            E1213 = Pattern.CommandE([ancilla[9], ancilla[10]])
-            E1314 = Pattern.CommandE([ancilla[10], ancilla[11]])
-            E1415 = Pattern.CommandE([ancilla[11], output2])
-            M1 = Pattern.CommandM(input1, 0, 'XY', [], [])
-            M2 = Pattern.CommandM(ancilla[0], pi / 2, 'XY', [], [])
-            M3 = Pattern.CommandM(ancilla[1], pi / 2, 'XY', [], [])
-            M4 = Pattern.CommandM(ancilla[2], pi / 2, 'XY', [], [])
-            M5 = Pattern.CommandM(ancilla[3], pi / 2, 'XY', [], [])
-            M6 = Pattern.CommandM(ancilla[4], pi / 2, 'XY', [], [])
-            M8 = Pattern.CommandM(ancilla[12], pi / 2, 'XY', [], [])
-            M12 = Pattern.CommandM(ancilla[9], pi / 2, 'XY', [], [])
-            M9 = Pattern.CommandM(input2, 0, 'XY', [], [])
-            M10 = Pattern.CommandM(ancilla[7], 0, 'XY', [], [])
-            M11 = Pattern.CommandM(ancilla[8], 0, 'XY', [], [])
-            M13 = Pattern.CommandM(ancilla[10], 0, 'XY', [], [])
-            M14 = Pattern.CommandM(ancilla[11], 0, 'XY', [], [])
-            M7 = Pattern.CommandM(ancilla[5], - pi, 'XY', [], input_ + [ancilla[i] for i in [1, 2, 3, 8, 12]])
-            M16 = Pattern.CommandM(ancilla[6], 0, 'XY', [], [ancilla[i] for i in [0, 1, 3, 4]])
-            X15 = Pattern.CommandX(output2, [ancilla[i] for i in [0, 1, 7, 9, 11, 12]])
-            X17 = Pattern.CommandX(output1, [ancilla[6]])
-            Z15 = Pattern.CommandZ(output2, [input2, ancilla[8], ancilla[10]])
-            Z17 = Pattern.CommandZ(output1, [ancilla[5]])
-            commands = [E12, E23, E34, E45, E48, E56, E67, E716, E1617, E910, E1011, E1112, E812, E1213, E1314, E1415,
-                        M1, M2, M3, M4, M5, M6, M8, M12, M9, M10, M11, M13, M14, M7, M16, X15, X17, Z15, Z17]
-
         elif name == 'cz':  # control Z gate
             commands = [Pattern.CommandE(input_)]
 
         elif name == 'm':  # single qubit measurement
-            self.__measured_qubits.append(which_qubit[0])
-            commands = [Pattern.CommandM(input_[0], *original_gate["basis"])]
+            self._measured_qubits.append(which_qubit[0])
+            commands = [Pattern.CommandM(input_[0],
+                                         original_gate["basis"]["angle"],
+                                         original_gate["basis"]["plane"],
+                                         original_gate["basis"]["domain_s"],
+                                         original_gate["basis"]["domain_t"])]
 
         else:
             raise ArgumentTypeError(f"Invalid gate: ({gate})!\n"
                                     f"Translation of such a gate is not supported in this version.")
 
-        self.__wild_pattern.append(
+        self._wild_pattern.append(
             Pattern(str([name] + [str(qubit) for qubit in which_qubit]), list(set(input_ + output_ + ancilla)),
                     input_, output_, commands))
 
-    def __list_to_net(self, vertices: list) -> List[List]:
+    def _list_to_net(self, vertices: list) -> List[List]:
         r"""Map a vertex list to a vertex net.
 
         This method maps a one-dimensional vertex list to a two-dimensional vertex net.
@@ -415,44 +369,46 @@ class MCalculus:
         """
         # Get width
         rows = {qubit[0] for qubit in vertices}
-        self.__circuit_width = len(rows) if self.__circuit_width is None else self.__circuit_width
+        self._circuit_width = len(rows) if self._circuit_width is None else self._circuit_width
         # Map to net
-        bit_net = [[] for _ in range(self.__circuit_width)]
+        bit_net = [[] for _ in range(self._circuit_width)]
         for qubit in vertices:
             bit_net[qubit[0]].append(qubit[1])
         return bit_net
 
-    def __get_input(self) -> List[Any]:
+    def _get_input(self) -> list:
         r"""Get the labels of the input vertices in the list.
 
         Returns:
             list: a list containing the labels of input vertices
         """
         input_list = []
-        for pat in self.__wild_pattern:
+        for pat in self._wild_pattern:
             input_list += pat.input_
         # Map to net
-        bit_net = self.__list_to_net(input_list)
-        input_ = [(i, min(bit_net[i])) for i in range(self.__circuit_width)]
+        bit_net = self._list_to_net(input_list)
+        input_ = [(i, min(bit_net[i])) for i in range(self._circuit_width)]
         return input_
 
-    def __get_output(self) -> List[Any]:
+    def _get_output(self) -> list:
         r"""Get the labels of the output vertices in the list.
 
         Returns:
             list: a list containing the labels of output vertices
         """
         output_list = []
-        for pat in self.__wild_pattern:
+        for pat in self._wild_pattern:
             output_list += pat.output_
-        bit_net = self.__list_to_net(output_list)
-        # Divide the output into classical channels and quantum channels
-        c_out = [(i, max(bit_net[i])) for i in range(self.__circuit_width) if i in self.__measured_qubits]
-        q_out = [(i, max(bit_net[i])) for i in range(self.__circuit_width) if i not in self.__measured_qubits]
+        bit_net = self._list_to_net(output_list)
+        # Divide the output into classical and quantum parts
+        c_out = [(i, max(bit_net[i]) if bit_net[i] != [] else 0) for i in range(self._circuit_width) if
+                 i in self._measured_qubits]
+        q_out = [(i, max(bit_net[i]) if bit_net[i] != [] else 0) for i in range(self._circuit_width) if
+                 i not in self._measured_qubits]
         output_ = [c_out, q_out]
         return output_
 
-    def __splice_patterns(self, input_: list, output_: list) -> None:
+    def _splice_patterns(self, input_: list, output_: list) -> None:
         r"""Splice all measurement patterns together to form a complete pattern.
 
         The spliced pattern is equivalent to the complete quantum circuit.
@@ -465,24 +421,24 @@ class MCalculus:
         space = []
         cmds = []
 
-        for pat in self.__wild_pattern:
+        for pat in self._wild_pattern:
             names += pat.name
             space += [qubit for qubit in pat.space if qubit not in space]
             cmds += pat.commands
 
-        self.__pattern = Pattern(names, space, input_, output_, cmds)
+        self._pattern = Pattern(names, space, input_, output_, cmds)
 
     def to_pattern(self) -> None:
         r"""Translate a quantum circuit to its equivalent measurement pattern.
         """
-        self.__slice_circuit(self.__circuit.gate_history)
-        for gate in self.__circuit_slice:
+        self.__slice_circuit(self._circuit.gate_history)
+        for gate in self._circuit_slice:
             self.__to_pattern(gate)
 
-        input_ = self.__get_input()
-        output_ = self.__get_output()
+        input_ = self._get_input()
+        output_ = self._get_output()
 
-        self.__splice_patterns(input_, output_)
+        self._splice_patterns(input_, output_)
 
     @staticmethod
     def __swap(two_cmds: list) -> list:
@@ -517,7 +473,7 @@ class MCalculus:
         # [X, E] --> [E, X]
         if name1 == 'X' and name2 == 'E':
             X_qubit = cmd1.which_qubit
-            E_qubits = cmd2.which_qubits[:]
+            E_qubits = cmd2.which_qubit[:]
             if X_qubit not in E_qubits:
                 return [cmd2, cmd1]  # swap the independent commands
             else:
@@ -531,7 +487,7 @@ class MCalculus:
 
         # [M, E] --> [E, M]
         elif name1 == 'M' and name2 == 'E':
-            if cmd1.which_qubit not in cmd2.which_qubits:
+            if cmd1.which_qubit not in cmd2.which_qubit:
                 return [cmd2, cmd1]  # swap the independent commands
             else:
                 raise ArgumentTypeError(f"Invalid command names: ({name1}) and ({name2})!"
@@ -541,6 +497,7 @@ class MCalculus:
         elif name1 == 'X' and name2 == 'M':
             X_qubit = cmd1.which_qubit
             M_qubit = cmd2.which_qubit
+
             if X_qubit != M_qubit:
                 return [cmd2, cmd1]  # swap the independent commands
             else:
@@ -559,6 +516,7 @@ class MCalculus:
         elif name1 == 'Z' and name2 == 'M':
             Z_qubit = cmd1.which_qubit
             M_qubit = cmd2.which_qubit
+
             if Z_qubit != M_qubit:
                 return [cmd2, cmd1]  # swap the independent commands
             else:
@@ -614,7 +572,7 @@ class MCalculus:
                 # command S and command M can swap independently without domain modification
                 return [cmd2, cmd1]  # swap the commands with domain modification
             # [S, X] --> [X, S], [S, Z] --> [Z, S], [S, S] --> [S, S]
-            elif name2 == 'X' or name2 == 'Z' or name2 == 'S':
+            elif name2 in ['X', 'Z', 'S']:
                 S_qubit = cmd1.which_qubit
                 S_domains = cmd1.domain
                 if S_qubit in cmd2.domain:
@@ -663,16 +621,15 @@ class MCalculus:
         return cmds
 
     @staticmethod
-    def __cmds_to_nums(cmds: list):
+    def __cmds_to_nums(cmds: list) -> Tuple[list, list, list]:
         r"""Map the commands to the numbers.
 
         Args:
             cmds (list): a list of commands
 
         Returns:
-            list: a list of counts on each type of the commands
-            list: a list of numbers
-            list: a list of numbers arranged from small to large
+            Tuple[list, list, list]: a tuple containing a list of counts on each type of the commands,
+            a list of numbers, and a list of numbers arranged from small to large
         """
         cmd_map = {'E': 1,  # order of commands E
                    'M': 2,  # order of commands M
@@ -722,7 +679,7 @@ class MCalculus:
 
         return cmd_num_wild[:pointer_map[cmd_type]] == cmd_num_std[:pointer_map[cmd_type]]
 
-    def __simplify_pauli_measurements(self):
+    def __simplify_pauli_measurements(self) -> None:
         r"""Simplify the dependencies of Pauli measurements.
 
         In certain cases, the dependencies to other vertices can be excluded with Pauli simplification method.
@@ -746,7 +703,7 @@ class MCalculus:
             \text{ are equal. Therefore, it has no dependencies to the vertices in } domain\_s
             \text{. So the } domain\_s \text{ can be excluded, and so on.}
         """
-        for cmd in self.__pattern.commands:
+        for cmd in self._pattern.commands:
             if cmd.name == 'M':
                 remainder = cmd.angle % (2 * pi)
                 if remainder in [0, pi]:
@@ -755,7 +712,7 @@ class MCalculus:
                     cmd.domain_t += cmd.domain_s[:]
                     cmd.domain_s = []
 
-    def standardize(self):
+    def standardize(self) -> None:
         r"""Standardize the measurement pattern.
 
         This method transforms a wild pattern to a standard 'EMC' pattern.
@@ -764,14 +721,14 @@ class MCalculus:
         To simplify the pattern, Pauli simplification method is automatically implemented to
         exclude some dependencies.
         """
-        cmds = self.__pattern.commands
+        cmds = self._pattern.commands
 
         for cmd_type in ['E', 'M', 'X', 'Z']:
             while not self.__is_standard(cmd_type, cmds):
                 cmds = self.__propagate(cmd_type, cmds)
-                print_progress(self.__distance_to_standard(cmds), "Standardization Progress", self.__track)
+                print_progress(self.__distance_to_standard(cmds), "Standardization Progress", self._track)
 
-        self.__pattern.commands = cmds
+        self._pattern.commands = cmds
         self.__simplify_pauli_measurements()
 
     @staticmethod
@@ -796,15 +753,15 @@ class MCalculus:
     def shift_signals(self) -> None:
         r"""Shift signal commands.
         """
-        cmds = self.__pattern.commands
+        cmds = self._pattern.commands
         cmds = self.__pull_out_domain_t(cmds)
 
         # Propagate CommandS
         while not self.__is_standard('S', cmds):
             cmds = self.__propagate('S', cmds)
-            print_progress(self.__distance_to_standard(cmds), "Signal Shifting Progress", self.__track)
+            print_progress(self.__distance_to_standard(cmds), "Signal Shifting Progress", self._track)
 
-        self.__pattern.commands = cmds
+        self._pattern.commands = cmds
 
     def get_pattern(self) -> "Pattern":
         r"""Get the measurement pattern.
@@ -812,44 +769,47 @@ class MCalculus:
         Returns:
             Pattern: a measurement pattern
         """
-        return self.__pattern
+        return self._pattern
 
-    def track_progress(self, track=True):
+    def track_progress(self, track=True) -> None:
         r"""Track the progress of measurement calculus.
 
         Args:
             track (bool, optional): whether to turn on the progress tracking
         """
-        self.__track = track
+        self._track = track
 
-    @staticmethod
-    def __default_order_by_row(labels: list) -> list:
-        r"""The default order of labels by row.
-
-        Ordering rule: smaller row/column number has a higher priority.
+    def optimize_by_row(self, order=None) -> None:
+        r"""Optimize the measurement order by the row-first principle.
 
         Args:
-            labels (list): a list of labels
-
-        Returns:
-            list: a list of ordered labels
-        """
-        # Construct a dict by string labels and their float values
-        labels_dict = {label: (label[0], label[1]) for label in labels}
-        # Sort the dict by values (sort row first and then column)
-        sorted_dict = dict(sorted(labels_dict.items(), key=lambda item: item[1]))
-        # Extract the keys in the dict
-        labels_sorted = list(sorted_dict.keys())
-
-        return labels_sorted
-
-    def optimize_by_row(self):
-        r"""Optimize the measurement orders by the row-first principle.
+            order (list, optional): the measurement order
 
         Warning:
             This is a heuristic algorithm that aims to measure each qubits row by row.
         """
-        cmds = self.__pattern.commands
+
+        def _default_order_by_row(labels: list, order: list) -> list:
+            r"""The default order of labels by row.
+
+            Ordering rule: smaller row/column number has a higher priority.
+
+            Args:
+                labels (list): a list of labels
+                order (list): the measurement order
+
+            Returns:
+                list: a list of ordered labels
+            """
+            container = {i: [] for i in order}
+            for label in labels:
+                container[label[0]].append(label)
+
+            return sum([container[item] for item in order], [])
+
+        order = list(range(self._circuit_width)) if order is None else order
+
+        cmds = self._pattern.commands
 
         # Split the commands by type
         cmdE_list = [cmd for cmd in cmds if cmd.name == 'E']
@@ -861,7 +821,7 @@ class MCalculus:
         cmdM_map = {cmd.which_qubit: cmd for cmd in cmdM_list}
 
         # Sort all the qubit labels by row
-        cmdM_qubit_list = self.__default_order_by_row([cmdM.which_qubit for cmdM in cmdM_list])
+        cmdM_qubit_list = _default_order_by_row([cmdM.which_qubit for cmdM in cmdM_list], order=order)
         mea_length = len(cmdM_qubit_list)
 
         for i in range(mea_length):
@@ -874,52 +834,85 @@ class MCalculus:
 
                 domains = set(cmdM_map[measuring].domain_s + cmdM_map[measuring].domain_t)
                 # Find the qubits in domain but not in front of the current measurement
-                push = self.__default_order_by_row(list(domains.difference(measured)))
+                push = _default_order_by_row(list(domains.difference(measured)), order=order)
                 if push:  # Remove qubits from the to_measure list and push it to the front
                     to_measure = [qubit for qubit in to_measure if qubit not in push]
                     cmdM_qubit_list = cmdM_qubit_list[:i] + push + [measuring] + to_measure
                 else:  # if no push qubits then jump out of the while loop
                     optimal = True
-            print_progress((i + 1) / mea_length, "Optimization Progress", self.__track)
+
+            print_progress((i + 1) / mea_length, "Optimization Progress", self._track)
 
         # Sort the measurement commands by the sorted qubit labels
         cmdM_opt = [cmdM_map[which_qubit] for which_qubit in cmdM_qubit_list]
 
         # Update pattern
         cmds = cmdE_list + cmdM_opt + cmdC_list + cmdS_list
-        self.__pattern.commands = cmds
+        self._pattern.commands = cmds
 
+    def optimize_by_column(self) -> None:
+        r"""Optimize the measurement orders by the column-first principle.
 
-def transpile_to_pattern(circuit: "Circuit", shift_signal=False, optimize=True, track=False) -> "Pattern":
-    r"""Transpile a quantum circuit to the equivalent measurement pattern.
+        Warning:
+            This is a heuristic algorithm that aims to measure each qubits column by column.
+        """
 
-    Args:
-        circuit (Circuit): quantum circuit to transpile
-        shift_signal (bool): whether to perform signal sifting after standardization
-        optimize (bool): whether to optimize the measurement orders
-        track (bool): whether to track the transpiling progress
+        def _default_order_by_column(labels: list) -> list:
+            r"""The default order of labels by column.
 
-    Warnings:
-        We should check if the circuit has sequential registers first.
-        If not, we need to perform remapping before transpiling the circuit.
+            Ordering rule: smaller column/row number has a higher priority.
 
-    Returns:
-        Pattern: an equivalent measurement pattern
-    """
-    # Check if circuit has sequential registers
-    if circuit.width != max(circuit.occupied_indices) + 1:
-        new_circuit = circuit.copy()
-        new_circuit.remap_indices()
-        circuit = new_circuit
+            Args:
+                labels (list): a list of labels
 
-    mc = MCalculus()
-    mc.track_progress(track)
-    mc.set_circuit(circuit)
-    mc.to_pattern()
-    mc.standardize()
-    if shift_signal:
-        mc.shift_signals()
-    if optimize:
-        mc.optimize_by_row()
-    pattern = mc.get_pattern()
-    return pattern
+            Returns:
+                list: a list of ordered labels
+            """
+            # Construct a dict by string labels and their float values
+            labels_dict = {label: (label[1], label[0]) for label in labels}
+            # Sort the dict by values (sort column first and then row)
+            sorted_dict = dict(sorted(labels_dict.items(), key=lambda item: item[1]))
+            # Extract the keys in the dict
+            labels_sorted = list(sorted_dict.keys())
+
+            return labels_sorted
+
+        cmds = self._pattern.commands
+
+        # Split the commands by type
+        cmdE_list = [cmd for cmd in cmds if cmd.name == 'E']
+        cmdM_list = [cmd for cmd in cmds if cmd.name == 'M']
+        cmdC_list = [cmd for cmd in cmds if cmd.name in ['X', 'Z']]
+        cmdS_list = [cmd for cmd in cmds if cmd.name == 'S']
+
+        # Construct a dict from qubit labels and their measurement commands
+        cmdM_map = {cmd.which_qubit: cmd for cmd in cmdM_list}
+
+        # Sort all the qubit labels by column
+        cmdM_qubit_list = _default_order_by_column([cmdM.which_qubit for cmdM in cmdM_list])
+        mea_length = len(cmdM_qubit_list)
+
+        for i in range(mea_length):
+            optimal = False
+            while not optimal:  # if the qubits list is not standard
+                # Slice measurement qubit list into three parts
+                measured = cmdM_qubit_list[:i]
+                measuring = cmdM_qubit_list[i]
+                to_measure = cmdM_qubit_list[i + 1:]
+
+                domains = set(cmdM_map[measuring].domain_s + cmdM_map[measuring].domain_t)
+                # Find the qubits in domain but not in front of the current measurement
+                push = _default_order_by_column(list(domains.difference(measured)))
+                if push:  # Remove qubits from the to_measure list and push it to the front
+                    to_measure = [qubit for qubit in to_measure if qubit not in push]
+                    cmdM_qubit_list = cmdM_qubit_list[:i] + push + [measuring] + to_measure
+                else:  # if no push qubits then jump out of the while loop
+                    optimal = True
+            print_progress((i + 1) / mea_length, "Optimization Progress", self._track)
+
+        # Sort the measurement commands by the sorted qubit labels
+        cmdM_opt = [cmdM_map[which_qubit] for which_qubit in cmdM_qubit_list]
+
+        # Update pattern
+        cmds = cmdE_list + cmdM_opt + cmdC_list + cmdS_list
+        self._pattern.commands = cmds
