@@ -119,14 +119,32 @@ class Network(Entity):
         nodes = []
         links = []
 
-        from qcompute_qnet.models.qkd.node import EndNode, TrustedRepeaterNode, BackboneNode
+        from qcompute_qnet.models.qkd.node import EndNode, TrustedRepeaterNode, BackboneNode,\
+            RMPRepeaterNode, RMPEndNode
         for node in topo['nodes']:
-            if node['type'] == "EndNode":
-                nodes.append(EndNode(node['name'], location=(node['longitude'], node['latitude'])))
+            if node['type'] == "RMPEndNode":
+                nodes.append(RMPEndNode(node['name'],
+                                        location=(node['longitude'], node['latitude'])
+                                        if node.get('longitude') and node.get('latitude') else None))
+            elif node['type'] == "EndNode":
+                nodes.append(EndNode(node['name'],
+                                     location=(node['longitude'], node['latitude'])
+                                     if node.get('longitude') and node.get('latitude') else None))
+            elif node['type'] == "RMPRepeaterNode":
+                nodes.append(RMPRepeaterNode(node['name'],
+                                             location=(node['longitude'], node['latitude'])
+                                             if node.get('longitude') and node.get('latitude') else None,
+                                             key_pool_volume=node.get('key_pool_volume', 20),
+                                             key_length=node.get('key_length', 32)))
             elif node['type'] == "TrustedRepeaterNode":
-                nodes.append(TrustedRepeaterNode(node['name'], location=(node['longitude'], node['latitude'])))
+                nodes.append(TrustedRepeaterNode(node['name'],
+                                                 location=(node['longitude'], node['latitude'])
+                                                 if node.get('longitude') and node.get('latitude') else None))
             elif node['type'] == "BackboneNode":
-                nodes.append(BackboneNode(node['name'], location=(node['longitude'], node['latitude'])))
+                nodes.append(BackboneNode(node['name'],
+                                          location=(node['longitude'], node['latitude'])
+                                          if node.get('longitude') and node.get('latitude') else None))
+
             else:
                 nodes.append(Node(node['name']))
 
@@ -144,7 +162,14 @@ class Network(Entity):
             ch.connect(n1, n2)
             # Set a duplex quantum channel
             q_distance = link['qchannel']['distance']
-            q_loss = link['qchannel']['loss'] if "loss" in link['qchannel'].keys() else None
+
+            if link['qchannel'].get('loss'):
+                q_loss = link['qchannel']['loss']
+            elif link['qchannel'].get('attenuation'):
+                q_loss = link['qchannel']['attenuation'] * q_distance * 1e-3
+            else:
+                q_loss = None
+
             q_delay = link['qchannel']['delay'] if "delay" in link['qchannel'].keys() else None
             qh = DuplexQuantumFiberChannel(f"q_{link['node1']}_{link['node2']}",
                                            distance=q_distance, loss=q_loss, delay=q_delay)
